
#include <LiquidCrystal_I2C.h>
// Instantiate the Objects
LiquidCrystal_I2C lcd(0x27, 16, 2);



void setup() {
  // put your setup code here, to run once:
  
  // Initialize the display
  //lcd.begin();
  lcd.backlight();
  lcd.cursor();

  UpdateDisplay();
  
}

void loop() {
  // put your main code here, to run repeatedly:

}


void UpdateDisplay()
{
  lcd.setCursor(0, 0);
  lcd.print("        ");
  lcd.setCursor(0, 0);
  lcd.setCursor(10, 0);
  lcd.print("ZL2CTM");

 
}
